import * as ScanditSDK from "scandit-sdk";

import OperationQueue from "./operation-queue";

export default class {
  operationQueue = new OperationQueue();
  isInitialized = false;

  initialize = (scannerContainer, props) => {
    ScanditSDK.BarcodePicker.create(scannerContainer, {
      accessCamera: props.accessCamera,
      camera: props.camera,
      cameraSettings: props.cameraSettings,
      cameraType: props.cameraType,
      enableCameraSwitcher: props.enableCameraSwitcher,
      enablePinchToZoom: props.enablePinchToZoom,
      enableTapToFocus: props.enableTapToFocus,
      enableTorchToggle: props.enableTorchToggle,
      guiStyle: props.guiStyle,
      laserArea: props.laserArea,
      playSoundOnScan: props.playSoundOnScan,
      scanningPaused: props.paused,
      scanSettings: props.scanSettings,
      singleImageModeSettings: props.singleImageModeSettings,
      targetScanningFPS: props.targetScanningFPS,
      vibrateOnScan: props.vibrateOnScan,
      videoFit: props.videoFit,
      visible: props.visible,
      viewfinderArea: props.viewfinderArea,
    }).then((picker) => {
      picker.on("ready", () => {
        props.onReady != null && props.onReady();
      });
      picker.on("scan", (result) => props.onScan != null && props.onScan(result));
      picker.on("scanError", (result) => props.onScanError != null && props.onScanError(result));
      picker.on("processFrame", (result) => props.onProcessFrame != null && props.onProcessFrame(result));
      picker.on("submitFrame", (result) => props.onSubmitFrame != null && props.onSubmitFrame(result));
      this.picker = picker;
      this.operationQueue.execute();
    });
    this.isInitialized = true;
  };

  destroy = () => {
    if (this.picker != null) {
      this.picker.destroy();
    }
  };

  addToQueue = (f) => {
    if (this.isInitialized == true) {
      this.operationQueue.add(f);
    }
  };

  update = (prevProps, props) => {
    this.setScanSettings(prevProps, props);
    this.setScanning(prevProps, props);
    this.setAccessCamera(prevProps, props);
    this.setActiveCamera(prevProps, props);
    this.setCameraSettings(prevProps, props);
    this.setEnableCameraSwitcher(prevProps, props);
    this.setEnableTorchToggle(prevProps, props);
    this.setEnablePinchToZoom(prevProps, props);
    this.setEnableTapToFocus(prevProps, props);
    this.setGuiStyle(prevProps, props);
    this.setLaserArea(prevProps, props);
    this.setMirrorImage(prevProps, props);
    this.setPlaySoundOnScanEnabled(prevProps, props);
    this.setVibrateOnScanEnabled(prevProps, props);
    this.setTargetScanningFPS(prevProps, props);
    this.setVideoFit(prevProps, props);
    this.setVisible(prevProps, props);
    this.setViewfinderArea(prevProps, props);
    this.setZoom(prevProps, props);

    if (this.picker != null) {
      this.operationQueue.execute();
    }
  };

  setScanSettings = (prevProps, props) => {
    const shouldSet = prevProps == null || JSON.stringify(prevProps.scanSettings) != JSON.stringify(props.scanSettings);

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.applyScanSettings(props.scanSettings)));
    }
  };

  setScanning = (prevProps, props) => {
    if (this.picker != null) {
      if (props.paused && !this.picker.isScanningPaused()) {
        // should pause?
        this.addToQueue(() => Promise.resolve(this.picker.pauseScanning(!props.accessCamera)));
      } else if (!props.paused && this.picker.isScanningPaused()) {
        // should resume?
        this.addToQueue(() => this.picker.resumeScanning());
      }
    }
  };

  setAccessCamera = (prevProps, props) => {
    const shouldSet = (prevProps == null || prevProps.accessCamera != props.accessCamera) && props.accessCamera;

    if (shouldSet) {
      this.addToQueue(() => this.picker.accessCamera());
    }
  };

  setActiveCamera = (prevProps, props) => {
    const shouldSet =
      (prevProps == null || prevProps.camera != props.camera) &&
      props.camera != null &&
      this.picker.getActiveCamera() != props.camera;

    if (shouldSet) {
      this.addToQueue(() => this.picker.setActiveCamera(props.camera));
    }
  };

  setCameraSettings = (prevProps, props) => {
    const shouldSet =
      prevProps == null || JSON.stringify(prevProps.cameraSettings) != JSON.stringify(props.cameraSettings);

    if (shouldSet) {
      this.addToQueue(() => this.picker.applyCameraSettings(props.cameraSettings));
    }
  };

  setEnableCameraSwitcher = (prevProps, props) => {
    const shouldSet = prevProps == null || prevProps.enableCameraSwitcher != props.enableCameraSwitcher;

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.setCameraSwitcherEnabled(props.enableCameraSwitcher)));
    }
  };

  setEnableTorchToggle = (prevProps, props) => {
    const shouldSet = prevProps == null || prevProps.enableTorchToggle != props.enableTorchToggle;

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.setTorchToggleEnabled(props.enableTorchToggle)));
    }
  };

  setEnablePinchToZoom = (prevProps, props) => {
    const shouldSet = prevProps == null || prevProps.enablePinchToZoom != props.enablePinchToZoom;

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.setPinchToZoomEnabled(props.enablePinchToZoom)));
    }
  };

  setEnableTapToFocus = (prevProps, props) => {
    const shouldSet = prevProps == null || prevProps.enableTapToFocus != props.enableTapToFocus;

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.setTapToFocusEnabled(props.enableTapToFocus)));
    }
  };

  setGuiStyle = (prevProps, props) => {
    const shouldSet = prevProps == null || prevProps.guiStyle != props.guiStyle;

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.setGuiStyle(props.guiStyle)));
    }
  };

  setLaserArea = (prevProps, props) => {
    const shouldSet = prevProps == null || JSON.stringify(prevProps.laserArea) != JSON.stringify(props.laserArea);

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.setLaserArea(props.laserArea)));
    }
  };

  setMirrorImage = (prevProps, props) => {
    const shouldSet = prevProps == null || prevProps.mirrorImage != props.mirrorImage;

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.setMirrorImageEnabled(props.mirrorImage)));
    }
  };

  setPlaySoundOnScanEnabled = (prevProps, props) => {
    const shouldSet = prevProps == null || prevProps.playSoundOnScan != props.playSoundOnScan;

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.setPlaySoundOnScanEnabled(props.playSoundOnScan)));
    }
  };

  setVibrateOnScanEnabled = (prevProps, props) => {
    const shouldSet = prevProps == null || prevProps.vibrateOnScan != props.vibrateOnScan;

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.setVibrateOnScanEnabled(props.vibrateOnScan)));
    }
  };

  setTargetScanningFPS = (prevProps, props) => {
    const shouldSet = prevProps == null || prevProps.targetScanningFPS != props.targetScanningFPS;

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.setTargetScanningFPS(props.targetScanningFPS)));
    }
  };

  setVideoFit = (prevProps, props) => {
    const shouldSet = prevProps == null || prevProps.videoFit != props.videoFit;

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.setVideoFit(props.videoFit)));
    }
  };

  setVisible = (prevProps, props) => {
    const shouldSet = prevProps == null || prevProps.visible != props.visible;

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.setVisible(props.visible)));
    }
  };

  setViewfinderArea = (prevProps, props) => {
    const shouldSet =
      prevProps == null || JSON.stringify(prevProps.viewfinderArea) != JSON.stringify(props.viewfinderArea);

    if (shouldSet) {
      this.addToQueue(() => Promise.resolve(this.picker.setViewfinderArea(props.viewfinderArea)));
    }
  };

  setZoom = (prevProps, props) => {
    const shouldSet = prevProps == null || prevProps.zoom != props.zoom;

    if (shouldSet) {
      this.addToQueue(() => this.picker.setZoom(props.zoom));
    }
  };
}
